use axum::{
    extract::{Path, State},
    routing::post,
    Router,
};
use axum_extra::{
    extract::{
        cookie::{Cookie, Expiration},
        CookieJar,
    },
    headers::{authorization::Basic, Authorization},
    TypedHeader,
};

use crate::{
    db::{Session, User},
    server::{
        error::{AppError, AppResult},
        Context,
    },
};

pub fn router() -> Router<Context> {
    Router::new().route("/{username}/login.json", post(post_login))
}

async fn post_login(
    State(ctx): State<Context>,
    Path(username): Path<String>,
    jar: CookieJar,
    TypedHeader(auth): TypedHeader<Authorization<Basic>>,
) -> AppResult<CookieJar> {
    // These should be the same according to the spec
    if username != auth.username() {
        return Err(AppError::BadRequest);
    }

    let session = tokio::task::spawn_blocking(move || {
        let user = User::by_username(&ctx.pool, auth.username())?.ok_or(AppError::NotFound)?;

        if user.verify_password(auth.password()) {
            Ok(Session::new_for_user(&ctx.pool, user.id)?)
        } else {
            Err(AppError::Unauthorized)
        }
    })
    .await
    .unwrap()?;

    Ok(jar.add(Cookie::build(("sessionid", session.id.to_string())).expires(Expiration::Session)))
}
